# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: acn.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database


# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\tacn.proto\x12\x12\x61\x65\x61.aea.acn.v1_0_0"\x92\x0b\n\nAcnMessage\x12P\n\x0c\x61\x65\x61_envelope\x18\x05 \x01(\x0b\x32\x38.aea.aea.acn.v1_0_0.AcnMessage.Aea_Envelope_PerformativeH\x00\x12T\n\x0elookup_request\x18\x06 \x01(\x0b\x32:.aea.aea.acn.v1_0_0.AcnMessage.Lookup_Request_PerformativeH\x00\x12V\n\x0flookup_response\x18\x07 \x01(\x0b\x32;.aea.aea.acn.v1_0_0.AcnMessage.Lookup_Response_PerformativeH\x00\x12H\n\x08register\x18\x08 \x01(\x0b\x32\x34.aea.aea.acn.v1_0_0.AcnMessage.Register_PerformativeH\x00\x12\x44\n\x06status\x18\t \x01(\x0b\x32\x32.aea.aea.acn.v1_0_0.AcnMessage.Status_PerformativeH\x00\x1a\xac\x01\n\x0b\x41gentRecord\x12\x12\n\nservice_id\x18\x01 \x01(\t\x12\x11\n\tledger_id\x18\x02 \x01(\t\x12\x0f\n\x07\x61\x64\x64ress\x18\x03 \x01(\t\x12\x12\n\npublic_key\x18\x04 \x01(\t\x12\x17\n\x0fpeer_public_key\x18\x05 \x01(\t\x12\x11\n\tsignature\x18\x06 \x01(\t\x12\x12\n\nnot_before\x18\x07 \x01(\t\x12\x11\n\tnot_after\x18\x08 \x01(\t\x1a\x92\x03\n\nStatusBody\x12\x46\n\x04\x63ode\x18\x01 \x01(\x0e\x32\x38.aea.aea.acn.v1_0_0.AcnMessage.StatusBody.StatusCodeEnum\x12\x0c\n\x04msgs\x18\x02 \x03(\t"\xad\x02\n\x0eStatusCodeEnum\x12\x0b\n\x07SUCCESS\x10\x00\x12\x1d\n\x19\x45RROR_UNSUPPORTED_VERSION\x10\x01\x12\x1c\n\x18\x45RROR_UNEXPECTED_PAYLOAD\x10\x02\x12\x11\n\rERROR_GENERIC\x10\x03\x12\x10\n\x0c\x45RROR_DECODE\x10\x04\x12\x1d\n\x19\x45RROR_WRONG_AGENT_ADDRESS\x10\n\x12\x1a\n\x16\x45RROR_WRONG_PUBLIC_KEY\x10\x0b\x12\x17\n\x13\x45RROR_INVALID_PROOF\x10\x0c\x12\x1c\n\x18\x45RROR_UNSUPPORTED_LEDGER\x10\r\x12\x1f\n\x1b\x45RROR_UNKNOWN_AGENT_ADDRESS\x10\x14\x12\x19\n\x15\x45RROR_AGENT_NOT_READY\x10\x15\x1aS\n\x15Register_Performative\x12:\n\x06record\x18\x01 \x01(\x0b\x32*.aea.aea.acn.v1_0_0.AcnMessage.AgentRecord\x1a\x34\n\x1bLookup_Request_Performative\x12\x15\n\ragent_address\x18\x01 \x01(\t\x1aZ\n\x1cLookup_Response_Performative\x12:\n\x06record\x18\x01 \x01(\x0b\x32*.aea.aea.acn.v1_0_0.AcnMessage.AgentRecord\x1ai\n\x19\x41\x65\x61_Envelope_Performative\x12\x10\n\x08\x65nvelope\x18\x01 \x01(\x0c\x12:\n\x06record\x18\x02 \x01(\x0b\x32*.aea.aea.acn.v1_0_0.AcnMessage.AgentRecord\x1aN\n\x13Status_Performative\x12\x37\n\x04\x62ody\x18\x01 \x01(\x0b\x32).aea.aea.acn.v1_0_0.AcnMessage.StatusBodyB\x0e\n\x0cperformativeb\x06proto3'
)


_ACNMESSAGE = DESCRIPTOR.message_types_by_name["AcnMessage"]
_ACNMESSAGE_AGENTRECORD = _ACNMESSAGE.nested_types_by_name["AgentRecord"]
_ACNMESSAGE_STATUSBODY = _ACNMESSAGE.nested_types_by_name["StatusBody"]
_ACNMESSAGE_REGISTER_PERFORMATIVE = _ACNMESSAGE.nested_types_by_name[
    "Register_Performative"
]
_ACNMESSAGE_LOOKUP_REQUEST_PERFORMATIVE = _ACNMESSAGE.nested_types_by_name[
    "Lookup_Request_Performative"
]
_ACNMESSAGE_LOOKUP_RESPONSE_PERFORMATIVE = _ACNMESSAGE.nested_types_by_name[
    "Lookup_Response_Performative"
]
_ACNMESSAGE_AEA_ENVELOPE_PERFORMATIVE = _ACNMESSAGE.nested_types_by_name[
    "Aea_Envelope_Performative"
]
_ACNMESSAGE_STATUS_PERFORMATIVE = _ACNMESSAGE.nested_types_by_name[
    "Status_Performative"
]
_ACNMESSAGE_STATUSBODY_STATUSCODEENUM = _ACNMESSAGE_STATUSBODY.enum_types_by_name[
    "StatusCodeEnum"
]
AcnMessage = _reflection.GeneratedProtocolMessageType(
    "AcnMessage",
    (_message.Message,),
    {
        "AgentRecord": _reflection.GeneratedProtocolMessageType(
            "AgentRecord",
            (_message.Message,),
            {
                "DESCRIPTOR": _ACNMESSAGE_AGENTRECORD,
                "__module__": "acn_pb2"
                # @@protoc_insertion_point(class_scope:aea.aea.acn.v1_0_0.AcnMessage.AgentRecord)
            },
        ),
        "StatusBody": _reflection.GeneratedProtocolMessageType(
            "StatusBody",
            (_message.Message,),
            {
                "DESCRIPTOR": _ACNMESSAGE_STATUSBODY,
                "__module__": "acn_pb2"
                # @@protoc_insertion_point(class_scope:aea.aea.acn.v1_0_0.AcnMessage.StatusBody)
            },
        ),
        "Register_Performative": _reflection.GeneratedProtocolMessageType(
            "Register_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ACNMESSAGE_REGISTER_PERFORMATIVE,
                "__module__": "acn_pb2"
                # @@protoc_insertion_point(class_scope:aea.aea.acn.v1_0_0.AcnMessage.Register_Performative)
            },
        ),
        "Lookup_Request_Performative": _reflection.GeneratedProtocolMessageType(
            "Lookup_Request_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ACNMESSAGE_LOOKUP_REQUEST_PERFORMATIVE,
                "__module__": "acn_pb2"
                # @@protoc_insertion_point(class_scope:aea.aea.acn.v1_0_0.AcnMessage.Lookup_Request_Performative)
            },
        ),
        "Lookup_Response_Performative": _reflection.GeneratedProtocolMessageType(
            "Lookup_Response_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ACNMESSAGE_LOOKUP_RESPONSE_PERFORMATIVE,
                "__module__": "acn_pb2"
                # @@protoc_insertion_point(class_scope:aea.aea.acn.v1_0_0.AcnMessage.Lookup_Response_Performative)
            },
        ),
        "Aea_Envelope_Performative": _reflection.GeneratedProtocolMessageType(
            "Aea_Envelope_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ACNMESSAGE_AEA_ENVELOPE_PERFORMATIVE,
                "__module__": "acn_pb2"
                # @@protoc_insertion_point(class_scope:aea.aea.acn.v1_0_0.AcnMessage.Aea_Envelope_Performative)
            },
        ),
        "Status_Performative": _reflection.GeneratedProtocolMessageType(
            "Status_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _ACNMESSAGE_STATUS_PERFORMATIVE,
                "__module__": "acn_pb2"
                # @@protoc_insertion_point(class_scope:aea.aea.acn.v1_0_0.AcnMessage.Status_Performative)
            },
        ),
        "DESCRIPTOR": _ACNMESSAGE,
        "__module__": "acn_pb2"
        # @@protoc_insertion_point(class_scope:aea.aea.acn.v1_0_0.AcnMessage)
    },
)
_sym_db.RegisterMessage(AcnMessage)
_sym_db.RegisterMessage(AcnMessage.AgentRecord)
_sym_db.RegisterMessage(AcnMessage.StatusBody)
_sym_db.RegisterMessage(AcnMessage.Register_Performative)
_sym_db.RegisterMessage(AcnMessage.Lookup_Request_Performative)
_sym_db.RegisterMessage(AcnMessage.Lookup_Response_Performative)
_sym_db.RegisterMessage(AcnMessage.Aea_Envelope_Performative)
_sym_db.RegisterMessage(AcnMessage.Status_Performative)

if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _ACNMESSAGE._serialized_start = 34
    _ACNMESSAGE._serialized_end = 1460
    _ACNMESSAGE_AGENTRECORD._serialized_start = 449
    _ACNMESSAGE_AGENTRECORD._serialized_end = 621
    _ACNMESSAGE_STATUSBODY._serialized_start = 624
    _ACNMESSAGE_STATUSBODY._serialized_end = 1026
    _ACNMESSAGE_STATUSBODY_STATUSCODEENUM._serialized_start = 725
    _ACNMESSAGE_STATUSBODY_STATUSCODEENUM._serialized_end = 1026
    _ACNMESSAGE_REGISTER_PERFORMATIVE._serialized_start = 1028
    _ACNMESSAGE_REGISTER_PERFORMATIVE._serialized_end = 1111
    _ACNMESSAGE_LOOKUP_REQUEST_PERFORMATIVE._serialized_start = 1113
    _ACNMESSAGE_LOOKUP_REQUEST_PERFORMATIVE._serialized_end = 1165
    _ACNMESSAGE_LOOKUP_RESPONSE_PERFORMATIVE._serialized_start = 1167
    _ACNMESSAGE_LOOKUP_RESPONSE_PERFORMATIVE._serialized_end = 1257
    _ACNMESSAGE_AEA_ENVELOPE_PERFORMATIVE._serialized_start = 1259
    _ACNMESSAGE_AEA_ENVELOPE_PERFORMATIVE._serialized_end = 1364
    _ACNMESSAGE_STATUS_PERFORMATIVE._serialized_start = 1366
    _ACNMESSAGE_STATUS_PERFORMATIVE._serialized_end = 1444
# @@protoc_insertion_point(module_scope)
