# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: contract_api.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database


# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x12\x63ontract_api.proto\x12\x1e\x61\x65\x61.valory.contract_api.v1_0_0"\x84\x11\n\x12\x43ontractApiMessage\x12V\n\x05\x65rror\x18\x05 \x01(\x0b\x32\x45.aea.valory.contract_api.v1_0_0.ContractApiMessage.Error_PerformativeH\x00\x12x\n\x16get_deploy_transaction\x18\x06 \x01(\x0b\x32V.aea.valory.contract_api.v1_0_0.ContractApiMessage.Get_Deploy_Transaction_PerformativeH\x00\x12j\n\x0fget_raw_message\x18\x07 \x01(\x0b\x32O.aea.valory.contract_api.v1_0_0.ContractApiMessage.Get_Raw_Message_PerformativeH\x00\x12r\n\x13get_raw_transaction\x18\x08 \x01(\x0b\x32S.aea.valory.contract_api.v1_0_0.ContractApiMessage.Get_Raw_Transaction_PerformativeH\x00\x12^\n\tget_state\x18\t \x01(\x0b\x32I.aea.valory.contract_api.v1_0_0.ContractApiMessage.Get_State_PerformativeH\x00\x12\x62\n\x0braw_message\x18\n \x01(\x0b\x32K.aea.valory.contract_api.v1_0_0.ContractApiMessage.Raw_Message_PerformativeH\x00\x12j\n\x0fraw_transaction\x18\x0b \x01(\x0b\x32O.aea.valory.contract_api.v1_0_0.ContractApiMessage.Raw_Transaction_PerformativeH\x00\x12V\n\x05state\x18\x0c \x01(\x0b\x32\x45.aea.valory.contract_api.v1_0_0.ContractApiMessage.State_PerformativeH\x00\x1a\x18\n\x06Kwargs\x12\x0e\n\x06kwargs\x18\x01 \x01(\x0c\x1a!\n\nRawMessage\x12\x13\n\x0braw_message\x18\x01 \x01(\x0c\x1a)\n\x0eRawTransaction\x12\x17\n\x0fraw_transaction\x18\x01 \x01(\x0c\x1a\x16\n\x05State\x12\r\n\x05state\x18\x01 \x01(\x0c\x1a\xaa\x01\n#Get_Deploy_Transaction_Performative\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x13\n\x0b\x63ontract_id\x18\x02 \x01(\t\x12\x10\n\x08\x63\x61llable\x18\x03 \x01(\t\x12I\n\x06kwargs\x18\x04 \x01(\x0b\x32\x39.aea.valory.contract_api.v1_0_0.ContractApiMessage.Kwargs\x1a\xc1\x01\n Get_Raw_Transaction_Performative\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x13\n\x0b\x63ontract_id\x18\x02 \x01(\t\x12\x18\n\x10\x63ontract_address\x18\x03 \x01(\t\x12\x10\n\x08\x63\x61llable\x18\x04 \x01(\t\x12I\n\x06kwargs\x18\x05 \x01(\x0b\x32\x39.aea.valory.contract_api.v1_0_0.ContractApiMessage.Kwargs\x1a\xbd\x01\n\x1cGet_Raw_Message_Performative\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x13\n\x0b\x63ontract_id\x18\x02 \x01(\t\x12\x18\n\x10\x63ontract_address\x18\x03 \x01(\t\x12\x10\n\x08\x63\x61llable\x18\x04 \x01(\t\x12I\n\x06kwargs\x18\x05 \x01(\x0b\x32\x39.aea.valory.contract_api.v1_0_0.ContractApiMessage.Kwargs\x1a\xb7\x01\n\x16Get_State_Performative\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x13\n\x0b\x63ontract_id\x18\x02 \x01(\t\x12\x18\n\x10\x63ontract_address\x18\x03 \x01(\t\x12\x10\n\x08\x63\x61llable\x18\x04 \x01(\t\x12I\n\x06kwargs\x18\x05 \x01(\x0b\x32\x39.aea.valory.contract_api.v1_0_0.ContractApiMessage.Kwargs\x1a]\n\x12State_Performative\x12G\n\x05state\x18\x01 \x01(\x0b\x32\x38.aea.valory.contract_api.v1_0_0.ContractApiMessage.State\x1az\n\x1cRaw_Transaction_Performative\x12Z\n\x0fraw_transaction\x18\x01 \x01(\x0b\x32\x41.aea.valory.contract_api.v1_0_0.ContractApiMessage.RawTransaction\x1an\n\x18Raw_Message_Performative\x12R\n\x0braw_message\x18\x01 \x01(\x0b\x32=.aea.valory.contract_api.v1_0_0.ContractApiMessage.RawMessage\x1an\n\x12\x45rror_Performative\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x13\n\x0b\x63ode_is_set\x18\x02 \x01(\x08\x12\x0f\n\x07message\x18\x03 \x01(\t\x12\x16\n\x0emessage_is_set\x18\x04 \x01(\x08\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\x42\x0e\n\x0cperformativeb\x06proto3'
)


_CONTRACTAPIMESSAGE = DESCRIPTOR.message_types_by_name["ContractApiMessage"]
_CONTRACTAPIMESSAGE_KWARGS = _CONTRACTAPIMESSAGE.nested_types_by_name["Kwargs"]
_CONTRACTAPIMESSAGE_RAWMESSAGE = _CONTRACTAPIMESSAGE.nested_types_by_name["RawMessage"]
_CONTRACTAPIMESSAGE_RAWTRANSACTION = _CONTRACTAPIMESSAGE.nested_types_by_name[
    "RawTransaction"
]
_CONTRACTAPIMESSAGE_STATE = _CONTRACTAPIMESSAGE.nested_types_by_name["State"]
_CONTRACTAPIMESSAGE_GET_DEPLOY_TRANSACTION_PERFORMATIVE = (
    _CONTRACTAPIMESSAGE.nested_types_by_name["Get_Deploy_Transaction_Performative"]
)
_CONTRACTAPIMESSAGE_GET_RAW_TRANSACTION_PERFORMATIVE = (
    _CONTRACTAPIMESSAGE.nested_types_by_name["Get_Raw_Transaction_Performative"]
)
_CONTRACTAPIMESSAGE_GET_RAW_MESSAGE_PERFORMATIVE = (
    _CONTRACTAPIMESSAGE.nested_types_by_name["Get_Raw_Message_Performative"]
)
_CONTRACTAPIMESSAGE_GET_STATE_PERFORMATIVE = _CONTRACTAPIMESSAGE.nested_types_by_name[
    "Get_State_Performative"
]
_CONTRACTAPIMESSAGE_STATE_PERFORMATIVE = _CONTRACTAPIMESSAGE.nested_types_by_name[
    "State_Performative"
]
_CONTRACTAPIMESSAGE_RAW_TRANSACTION_PERFORMATIVE = (
    _CONTRACTAPIMESSAGE.nested_types_by_name["Raw_Transaction_Performative"]
)
_CONTRACTAPIMESSAGE_RAW_MESSAGE_PERFORMATIVE = _CONTRACTAPIMESSAGE.nested_types_by_name[
    "Raw_Message_Performative"
]
_CONTRACTAPIMESSAGE_ERROR_PERFORMATIVE = _CONTRACTAPIMESSAGE.nested_types_by_name[
    "Error_Performative"
]
ContractApiMessage = _reflection.GeneratedProtocolMessageType(
    "ContractApiMessage",
    (_message.Message,),
    {
        "Kwargs": _reflection.GeneratedProtocolMessageType(
            "Kwargs",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTRACTAPIMESSAGE_KWARGS,
                "__module__": "contract_api_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.contract_api.v1_0_0.ContractApiMessage.Kwargs)
            },
        ),
        "RawMessage": _reflection.GeneratedProtocolMessageType(
            "RawMessage",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTRACTAPIMESSAGE_RAWMESSAGE,
                "__module__": "contract_api_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.contract_api.v1_0_0.ContractApiMessage.RawMessage)
            },
        ),
        "RawTransaction": _reflection.GeneratedProtocolMessageType(
            "RawTransaction",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTRACTAPIMESSAGE_RAWTRANSACTION,
                "__module__": "contract_api_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.contract_api.v1_0_0.ContractApiMessage.RawTransaction)
            },
        ),
        "State": _reflection.GeneratedProtocolMessageType(
            "State",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTRACTAPIMESSAGE_STATE,
                "__module__": "contract_api_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.contract_api.v1_0_0.ContractApiMessage.State)
            },
        ),
        "Get_Deploy_Transaction_Performative": _reflection.GeneratedProtocolMessageType(
            "Get_Deploy_Transaction_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTRACTAPIMESSAGE_GET_DEPLOY_TRANSACTION_PERFORMATIVE,
                "__module__": "contract_api_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.contract_api.v1_0_0.ContractApiMessage.Get_Deploy_Transaction_Performative)
            },
        ),
        "Get_Raw_Transaction_Performative": _reflection.GeneratedProtocolMessageType(
            "Get_Raw_Transaction_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTRACTAPIMESSAGE_GET_RAW_TRANSACTION_PERFORMATIVE,
                "__module__": "contract_api_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.contract_api.v1_0_0.ContractApiMessage.Get_Raw_Transaction_Performative)
            },
        ),
        "Get_Raw_Message_Performative": _reflection.GeneratedProtocolMessageType(
            "Get_Raw_Message_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTRACTAPIMESSAGE_GET_RAW_MESSAGE_PERFORMATIVE,
                "__module__": "contract_api_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.contract_api.v1_0_0.ContractApiMessage.Get_Raw_Message_Performative)
            },
        ),
        "Get_State_Performative": _reflection.GeneratedProtocolMessageType(
            "Get_State_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTRACTAPIMESSAGE_GET_STATE_PERFORMATIVE,
                "__module__": "contract_api_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.contract_api.v1_0_0.ContractApiMessage.Get_State_Performative)
            },
        ),
        "State_Performative": _reflection.GeneratedProtocolMessageType(
            "State_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTRACTAPIMESSAGE_STATE_PERFORMATIVE,
                "__module__": "contract_api_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.contract_api.v1_0_0.ContractApiMessage.State_Performative)
            },
        ),
        "Raw_Transaction_Performative": _reflection.GeneratedProtocolMessageType(
            "Raw_Transaction_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTRACTAPIMESSAGE_RAW_TRANSACTION_PERFORMATIVE,
                "__module__": "contract_api_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.contract_api.v1_0_0.ContractApiMessage.Raw_Transaction_Performative)
            },
        ),
        "Raw_Message_Performative": _reflection.GeneratedProtocolMessageType(
            "Raw_Message_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTRACTAPIMESSAGE_RAW_MESSAGE_PERFORMATIVE,
                "__module__": "contract_api_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.contract_api.v1_0_0.ContractApiMessage.Raw_Message_Performative)
            },
        ),
        "Error_Performative": _reflection.GeneratedProtocolMessageType(
            "Error_Performative",
            (_message.Message,),
            {
                "DESCRIPTOR": _CONTRACTAPIMESSAGE_ERROR_PERFORMATIVE,
                "__module__": "contract_api_pb2"
                # @@protoc_insertion_point(class_scope:aea.valory.contract_api.v1_0_0.ContractApiMessage.Error_Performative)
            },
        ),
        "DESCRIPTOR": _CONTRACTAPIMESSAGE,
        "__module__": "contract_api_pb2"
        # @@protoc_insertion_point(class_scope:aea.valory.contract_api.v1_0_0.ContractApiMessage)
    },
)
_sym_db.RegisterMessage(ContractApiMessage)
_sym_db.RegisterMessage(ContractApiMessage.Kwargs)
_sym_db.RegisterMessage(ContractApiMessage.RawMessage)
_sym_db.RegisterMessage(ContractApiMessage.RawTransaction)
_sym_db.RegisterMessage(ContractApiMessage.State)
_sym_db.RegisterMessage(ContractApiMessage.Get_Deploy_Transaction_Performative)
_sym_db.RegisterMessage(ContractApiMessage.Get_Raw_Transaction_Performative)
_sym_db.RegisterMessage(ContractApiMessage.Get_Raw_Message_Performative)
_sym_db.RegisterMessage(ContractApiMessage.Get_State_Performative)
_sym_db.RegisterMessage(ContractApiMessage.State_Performative)
_sym_db.RegisterMessage(ContractApiMessage.Raw_Transaction_Performative)
_sym_db.RegisterMessage(ContractApiMessage.Raw_Message_Performative)
_sym_db.RegisterMessage(ContractApiMessage.Error_Performative)

if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _CONTRACTAPIMESSAGE._serialized_start = 55
    _CONTRACTAPIMESSAGE._serialized_end = 2235
    _CONTRACTAPIMESSAGE_KWARGS._serialized_start = 903
    _CONTRACTAPIMESSAGE_KWARGS._serialized_end = 927
    _CONTRACTAPIMESSAGE_RAWMESSAGE._serialized_start = 929
    _CONTRACTAPIMESSAGE_RAWMESSAGE._serialized_end = 962
    _CONTRACTAPIMESSAGE_RAWTRANSACTION._serialized_start = 964
    _CONTRACTAPIMESSAGE_RAWTRANSACTION._serialized_end = 1005
    _CONTRACTAPIMESSAGE_STATE._serialized_start = 1007
    _CONTRACTAPIMESSAGE_STATE._serialized_end = 1029
    _CONTRACTAPIMESSAGE_GET_DEPLOY_TRANSACTION_PERFORMATIVE._serialized_start = 1032
    _CONTRACTAPIMESSAGE_GET_DEPLOY_TRANSACTION_PERFORMATIVE._serialized_end = 1202
    _CONTRACTAPIMESSAGE_GET_RAW_TRANSACTION_PERFORMATIVE._serialized_start = 1205
    _CONTRACTAPIMESSAGE_GET_RAW_TRANSACTION_PERFORMATIVE._serialized_end = 1398
    _CONTRACTAPIMESSAGE_GET_RAW_MESSAGE_PERFORMATIVE._serialized_start = 1401
    _CONTRACTAPIMESSAGE_GET_RAW_MESSAGE_PERFORMATIVE._serialized_end = 1590
    _CONTRACTAPIMESSAGE_GET_STATE_PERFORMATIVE._serialized_start = 1593
    _CONTRACTAPIMESSAGE_GET_STATE_PERFORMATIVE._serialized_end = 1776
    _CONTRACTAPIMESSAGE_STATE_PERFORMATIVE._serialized_start = 1778
    _CONTRACTAPIMESSAGE_STATE_PERFORMATIVE._serialized_end = 1871
    _CONTRACTAPIMESSAGE_RAW_TRANSACTION_PERFORMATIVE._serialized_start = 1873
    _CONTRACTAPIMESSAGE_RAW_TRANSACTION_PERFORMATIVE._serialized_end = 1995
    _CONTRACTAPIMESSAGE_RAW_MESSAGE_PERFORMATIVE._serialized_start = 1997
    _CONTRACTAPIMESSAGE_RAW_MESSAGE_PERFORMATIVE._serialized_end = 2107
    _CONTRACTAPIMESSAGE_ERROR_PERFORMATIVE._serialized_start = 2109
    _CONTRACTAPIMESSAGE_ERROR_PERFORMATIVE._serialized_end = 2219
# @@protoc_insertion_point(module_scope)
