@font-face {
    font-family: 'triakis__fontbold';
    src: url('triakisfont-bold-webfont.woff2') format('woff2'),
        url('triakisfont-bold-webfont.woff') format('woff');
    font-weight: normal;
    font-style: normal;
    font-display: swap;
}

@font-face {
    font-family: 'triakis__fontbold-italic';
    src: url('triakisfont-bolditalic-webfont.woff2') format('woff2'),
        url('triakisfont-bolditalic-webfont.woff') format('woff');
    font-weight: normal;
    font-style: normal;
    font-display: swap;
}

@font-face {
    font-family: 'triakis__fontheavy';
    src: url('triakisfont-heavy-webfont.woff2') format('woff2'),
        url('triakisfont-heavy-webfont.woff') format('woff');
    font-weight: normal;
    font-style: normal;
    font-display: swap;
}

@font-face {
    font-family: 'triakis__fontheavy_italic';
    src: url('triakisfont-heavy-italic-webfont.woff2') format('woff2'),
        url('triakisfont-heavy-italic-webfont.woff') format('woff');
    font-weight: normal;
    font-style: normal;
    font-display: swap;
}

@font-face {
    font-family: 'triakis__fontlight';
    src: url('triakisfont-light-webfont.woff2') format('woff2'),
        url('triakisfont-light-webfont.woff') format('woff');
    font-weight: normal;
    font-style: normal;
    font-display: swap;
}

@font-face {
    font-family: 'triakis__fontlight_italic';
    src: url('triakisfont-light-italic-webfont.woff2') format('woff2'),
        url('triakisfont-light-italic-webfont.woff') format('woff');
    font-weight: normal;
    font-style: normal;
    font-display: swap;
}

@font-face {
    font-family: 'triakis__fontmedium';
    src: url('triakisfont-medium-webfont.woff2') format('woff2'),
        url('triakisfont-medium-webfont.woff') format('woff');
    font-weight: normal;
    font-style: normal;
    font-display: swap;
}

@font-face {
    font-family: 'triakis__fontmedium-italic';
    src: url('triakisfont-mediumitalic-webfont.woff2') format('woff2'),
        url('triakisfont-mediumitalic-webfont.woff') format('woff');
    font-weight: normal;
    font-style: normal;
    font-display: swap;
}

@font-face {
    font-family: 'triakis__font_lightregular';
    src: url('triakisfont-regular-webfont.woff2') format('woff2'),
        url('triakisfont-regular-webfont.woff') format('woff');
    font-weight: normal;
    font-style: normal;
    font-display: swap;
}

@font-face {
    font-family: 'triakis__fontsemibold';
    src: url('triakisfont-semibold-webfont.woff2') format('woff2'),
        url('triakisfont-semibold-webfont.woff') format('woff');
    font-weight: normal;
    font-style: normal;
    font-display: swap;
}
